Type: Package
Package: gimap
Title: Calculate Genetic Interactions for Paired CRISPR Targets
Version: 1.1.2
Authors@R: c(
  person("Candace", "Savonen",
         email = c("cansav09@gmail.com","csavonen@fredhutch.org"), role = c("aut", "cre")),
  person("Phoebe", "Parrish",
         email = "pcrparrish@gmail.com", role = c("aut")),
  person("Kate", "Isaac",
         email = "kisaac@fredhutch.org", role = c("aut")),
  person("Howard", "Baek",
         email = "howardbaek.fh@gmail.com", role = c("aut")),
  person("Daniel", "Grosso", role = c("aut")),
  person("Siobhan", "O'Brien", role = c("aut")),
  person("Alice", "Berger", role = c("aut")))
Author: Candace Savonen [aut, cre],
  Phoebe Parrish [aut],
  Kate Isaac [aut],
  Howard Baek [aut],
  Daniel Grosso [aut],
  Siobhan O'Brien [aut],
  Alice Berger [aut]
Maintainer: Candace Savonen <cansav09@gmail.com>
Description: Helps find meaningful patterns in complex genetic experiments. First gimap takes data from paired CRISPR (Clustered regularly interspaced short palindromic repeats) screens that has been pre-processed to counts table of paired gRNA (guide Ribonucleic Acid) reads. The input data will have cell counts for how well cells grow (or don't grow) when different genes or pairs of genes are disabled. The output of the 'gimap' package is genetic interaction scores which are  the distance between the observed CRISPR score and the expected CRISPR score. The expected CRISPR scores are what we expect for the CRISPR values to be for two unrelated genes. The further away an observed CRISPR score is from its expected score the more we suspect genetic interaction. The work in this package is based off of original research from the Alice Berger lab at Fred Hutchinson Cancer Center (2021) <doi:10.1016/j.celrep.2021.109597>.
License: GPL-3
URL: https://github.com/FredHutch/gimap
BugReports: https://github.com/FredHutch/gimap/issues
Imports: readr, dplyr, tidyr, rmarkdown, vroom, ggplot2, magrittr,
        pheatmap, purrr, janitor, utils, stats, stringr, httr,
        jsonlite, openssl
Suggests: testthat (>= 3.0.0), roxygen2, kableExtra, knitr
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.2
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-02-05 16:36:51 UTC; candace
Depends: R (>= 3.5.0)
Repository: CRAN
Date/Publication: 2026-02-05 22:10:02 UTC
Built: R 4.6.0; ; 2026-04-26 18:43:30 UTC; unix
