# Generated by roxygen2: do not edit by hand

export("%>%")
export(calc_gi)
export(delete_annotation)
export(delete_example_data)
export(example_data_folder)
export(get_example_data)
export(get_figshare)
export(gimap_annotate)
export(gimap_filter)
export(gimap_normalize)
export(gimap_object)
export(plot_crispr)
export(plot_exp_v_obs_scatter)
export(plot_rank_scatter)
export(plot_targets)
export(plot_theme)
export(plot_volcano)
export(qc_cdf)
export(qc_constructs_countzero_bar)
export(qc_cor_heatmap)
export(qc_filter_plasmid)
export(qc_filter_zerocounts)
export(qc_plasmid_histogram)
export(qc_sample_hist)
export(qc_variance_hist)
export(run_qc)
export(save_example_timepoint_data)
export(save_example_treatment_data)
export(setup_data)
export(supported_cell_lines)
import(dplyr)
import(ggplot2)
import(httr)
importFrom(dplyr,across)
importFrom(dplyr,if_any)
importFrom(dplyr,mutate)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(janitor,clean_names)
importFrom(jsonlite,fromJSON)
importFrom(magrittr,"%>%")
importFrom(openssl,aes_cbc_decrypt)
importFrom(pheatmap,pheatmap)
importFrom(purrr,map)
importFrom(purrr,reduce)
importFrom(stats,cor)
importFrom(stats,lm)
importFrom(stats,p.adjust)
importFrom(stats,quantile)
importFrom(stats,t.test)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(stringr,word)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,unite)
importFrom(utils,browseURL)
importFrom(utils,download.file)
importFrom(utils,installed.packages)
importFrom(utils,menu)
importFrom(utils,unzip)
