% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{figshare_manual_download_message}
\alias{figshare_manual_download_message}
\title{Build manual download instructions for Figshare files}
\usage{
figshare_manual_download_message(item, files)
}
\arguments{
\item{item}{Figshare item ID.}

\item{files}{Character vector of file names.}
}
\value{
A formatted message string.
}
\description{
Internal helper to build a warning message for manual downloads.
}
\keyword{internal}
