% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{example_data_folder}
\alias{example_data_folder}
\title{Folder used for Figshare example data}
\usage{
example_data_folder(data_dir = NULL)
}
\arguments{
\item{data_dir}{Optional directory. If `NULL`, returns the session example
cache under [tempdir()] (same default as [get_example_data()]). Otherwise
resolves via [gimap_data_dir()].}
}
\value{
Normalized path to the folder where example files are stored.
}
\description{
Folder used for Figshare example data
}
